@@ -19,15 +19,12 @@
 
 #include "omalloc/omalloc.h"
 #include "misc/options.h"
+
 #include "polys/monomials/p_polys.h"
 #include "polys/kbuckets.h"
 
 #include "kernel/polys.h"
 
-#ifdef HAVE_SHIFTBBA
-#include "polys/shiftop.h"
-#endif
-
 
 #define HAVE_TAIL_BIN
 // This doesn't really work, fixme, if necessary
@@ -122,32 +119,12 @@ KINLINE void sTObject::Set(poly p_in, ring r)
   if (r != currRing)
   {
     assume(r == tailRing);
-#ifdef HAVE_SHIFTBBA
-    if (r->isLPring)
-    {
-      shift = p_mFirstVblock(p_in, r);
-      if (!shift) p_Test(p_in, r);
-    }
-    else
-#endif
-    {
-      p_Test(p_in, r);
-    }
+    p_Test(p_in, r);
     t_p = p_in;
   }
   else
   {
-#ifdef HAVE_SHIFTBBA
-    if (currRing->isLPring)
-    {
-      shift = p_mFirstVblock(p_in, currRing);
-      if (!shift) p_Test(p_in, currRing);
-    }
-    else
-#endif
-    {
-      p_Test(p_in, currRing);
-    }
+    p_Test(p_in, currRing);
     p = p_in;
   }
   pLength=::pLength(p_in);
@@ -164,17 +141,7 @@ KINLINE void sTObject::Set(poly p_in, ring c_r, ring t_r)
   if (c_r != t_r)
   {
     assume(c_r == currRing && t_r == tailRing);
-#ifdef HAVE_SHIFTBBA
-    if (c_r->isLPring)
-    {
-      shift = p_mFirstVblock(p_in, c_r);
-      if (!shift) p_Test(p_in, currRing);
-    }
-    else
-#endif
-    {
-      p_Test(p_in, currRing);
-    }
+    p_Test(p_in, currRing);
     p = p_in;
     pLength=::pLength(p_in);
   }
@@ -239,7 +206,8 @@ KINLINE void sTObject::Copy()
     t_p = p_Copy(t_p, tailRing);
     if (p != NULL) /* and t_p!=NULL*/
     {
-      p = p_LmInit(p, currRing);
+      p = p_Head(p, currRing);
+      n_Delete(&pGetCoeff(p),currRing->cf);
       pGetCoeff(p)=pGetCoeff(t_p);
       pNext(p) = pNext(t_p);
     }
@@ -366,13 +334,6 @@ KINLINE void sLObject::Normalize()
   {
     pNormalize(p);
   }
-  if (bucket!=NULL) kBucketNormalize(bucket);
-}
-
-KINLINE void sLObject::CanonicalizeP()
-{
-  if (bucket != NULL)
-    kBucketCanonicalize(bucket);
 }
 
 KINLINE void sLObject::HeadNormalize()
@@ -679,6 +640,21 @@ KINLINE poly sLObject::LmExtractAndIter()
   return ret;
 }
 
+KINLINE poly sLObject::CanonicalizeP()
+{
+  //kTest_L(this);
+  int i = -1;
+
+  if (bucket != NULL)
+    i = kBucketCanonicalize(bucket);
+
+  if (p == NULL)
+    p = k_LmInit_tailRing_2_currRing(t_p, tailRing);
+
+  if (i >= 0) pNext(p) = bucket->buckets[i];
+  return p;
+}
+
 KINLINE poly sLObject::GetTP()
 {
   //kTest_L(this);
@@ -782,7 +758,7 @@ KINLINE long sLObject::pLDeg(BOOLEAN deg_last)
 
   long ldeg;
   ldeg = tailRing->pLDeg(GetLmTailRing(), &length, tailRing);
-#ifndef SING_NDEBUG
+#ifdef HAVE_ASSUME
   if ( pLength == 0)
     p_Last(GetLmTailRing(), pLength, tailRing);
   assume ( pLength == length || rIsSyzIndexRing(currRing));
@@ -1052,44 +1028,6 @@ KINLINE void k_GetStrongLeadTerms(const poly p1, const poly p2, const ring leadR
  * Misc things
  *
  ***************************************************************/
-KINLINE int ksReducePolyTailLC_Z(LObject* PR, TObject* PW, LObject* Red)
-{
-  BOOLEAN ret;
-  number mult, rest;
-  TObject red = *PW;
-  red.Copy();
-  rest = n_QuotRem(pGetCoeff(Red->p), pGetCoeff(red.p),
-          &mult, currRing->cf);
-  red.Mult_nn(rest);
-
-  assume(PR->GetLmCurrRing() != red.GetLmCurrRing());
-  ret = ksReducePolyLC(Red, &red, NULL, &mult);
-  red.Delete();
-  red.Clear();
-
-  return ret;
-}
-
-KINLINE int ksReducePolyTail_Z(LObject* PR, TObject* PW, LObject* Red)
-{
-  BOOLEAN ret;
-  number coef;
-
-  assume(PR->GetLmCurrRing() != PW->GetLmCurrRing());
-  ret = ksReducePoly(Red, PW, NULL, &coef);
-
-  if (!ret)
-  {
-    if (! n_IsOne(coef, currRing->cf))
-    {
-      PR->Mult_nn(coef);
-      // HANNES: mark for Normalize
-    }
-    n_Delete(&coef, currRing->cf);
-  }
-  return ret;
-}
-
 KINLINE int ksReducePolyTail(LObject* PR, TObject* PW, LObject* Red)
 {
   BOOLEAN ret;
