@@ -84,7 +84,7 @@
 #define ALIGN_8
 #endif
 
-#define SINGULAR_PATCHLEVEL 3
+#define SINGULAR_PATCHLEVEL 0
 #define SINGULAR_VERSION ((SINGULAR_MAJOR_VERSION*1000 + SINGULAR_MINOR_VERSION*100 + SINGULAR_SUB_VERSION*10)+SINGULAR_PATCHLEVEL)
 
 /*******************************************************************
@@ -133,6 +133,17 @@ static inline void HALT() { m2_end(2);}
 /* #define MAX_INT_LEN 11 */
 
 
+#ifdef DO_PROFILE
+/* define to enable explicit profiling of some crucial inline
+ * routines and defines  */
+#undef DO_DEEP_PROFILE
+#endif
+
+/* define to enable assume */
+#ifndef HAVE_ASSUME
+#undef HAVE_ASSUME
+#endif
+
 /* define LINKAGE to "extern C" if compiling for shared libs */
 #ifndef LINKAGE
 #if defined(PIC)
@@ -153,6 +164,9 @@ static inline void HALT() { m2_end(2);}
 /* undefine to enable inline */
 #define NO_INLINE
 
+/* undefine to disable assume -- should normally be defined for SING_NDEBUG */
+#define HAVE_ASSUME
+
 /* undef PDEBUG to disable checks of polys
 
  define PDEBUG to
@@ -379,10 +393,10 @@ extern void dErrorBreak();
 }
 #endif
 
-#ifdef SING_NDEBUG
+#ifndef HAVE_ASSUME
 #define assume(x) do {} while (0)
 #define r_assume(x) do {} while (0)
-#else /* SING_NDEBUG */
+#else /* ! HAVE_ASSUME */
 
 #define assume_violation(s,f,l) \
   dReportError("assume violation at %s:%d condition: %s", f,l,s)
@@ -410,10 +424,10 @@ do                                              \
   }                                             \
 }                                               \
 while (0)
-#endif /* SING_NDEBUG */
+#endif /* HAVE_ASSUME */
 
 /* do have RDEBUG, unless we are doing the very real thing */
-#ifndef SING_NDEBUG
+#ifdef HAVE_ASSUME
 #ifndef RDEBUG
 #define RDEBUG
 #endif
